use Test::More;

use_ok 'Cog';
use_ok 'Cog::App';
use_ok 'Cog::App::FileBrowser';
use_ok 'Cog::Base';
use_ok 'Cog::Config';
use_ok 'Cog::Maker';
use_ok 'Cog::Plugin';
use_ok 'Cog::Runner';
use_ok 'Cog::WebApp';

done_testing;
