package Cog::Plugin;
use Mo;

sub plugins { [] }

sub url_map { [] }
sub site_navigation { [] }

sub coffee_files { [] }
sub js_files { [] }
sub css_files { [] }
sub image_files { [] }
sub template_files { [] }

1;
